@@ -54,6 +54,8 @@ command_input(struct confmodule *mod, char *arg)
 
     if (visible)
         visible = mod->frontend->methods.add(mod->frontend, q);
+    else
+        mod->frontend->methods.add_noninteractive(mod->frontend, q);
 
     if (q->priority != NULL)
         free(q->priority);
@@ -127,9 +129,12 @@ command_capb(struct confmodule *mod, char *arg)
             mod->frontend->capability |= DCF_CAPB_BACKUP;
         else if (strcmp(argv[i], "progresscancel") == 0)
             mod->frontend->capability |= DCF_CAPB_PROGRESSCANCEL;
+        else if (strcmp(argv[i], "align") == 0)
+            mod->frontend->capability |= DCF_CAPB_ALIGN;
     }
 
-    if (asprintf(&out, "%u multiselect backup progresscancel", CMDSTATUS_SUCCESS) == -1)
+    if (asprintf(&out, "%u multiselect backup progresscancel align",
+                 CMDSTATUS_SUCCESS) == -1)
         DIE("Out of memory");
 
     plugin_state = NULL;
@@ -148,7 +153,7 @@ command_capb(struct confmodule *mod, char *arg)
         out = newout;
         outend = mempcpy(outend, " plugin-", 8);
         outend = mempcpy(outend, plugin->name, namelen);
-        *outend++ = '\0';
+        *outend = '\0';
     }
 
     return out;
@@ -159,9 +164,6 @@ command_title(struct confmodule *mod, char *arg)
 {
     char *out;
 
-    if (*arg)
-        debug_printf(0, "Obsolete command TITLE %s called", arg);
-
     mod->frontend->methods.set_title(mod->frontend, arg);
     asprintf(&out, "%u", CMDSTATUS_SUCCESS);
     return out;
@@ -201,7 +203,7 @@ command_go(struct confmodule *mod, char *arg)
 
     q = mod->questions->methods.get(mod->questions, "debconf/frontend");
     if (q)
-	requested_frontend = question_getvalue(q, "");
+	requested_frontend = question_getvalue(q, "C");
     question_deref(q);
 
     running_frontend = getenv("DEBIAN_FRONTEND");
@@ -217,6 +219,7 @@ command_go(struct confmodule *mod, char *arg)
 	mod->frontend->questions = q;
     }
 
+    mod->frontend->methods.go_noninteractive(mod->frontend);
     ret = mod->frontend->methods.go(mod->frontend);
     if (ret == CMDSTATUS_GOBACK || mod->backed_up != 0)
     {
@@ -258,7 +261,7 @@ command_get(struct confmodule *mod, char *arg)
                 CMDSTATUS_BADQUESTION, argv[0]);
     else 
     {
-        const char *value = question_getvalue(q, NULL);
+        const char *value = question_getvalue(q, "C");
         asprintf(&out, "%u %s",
                 CMDSTATUS_SUCCESS, value ? value : "");
     }
@@ -284,6 +287,7 @@ command_set(struct confmodule *mod, char *arg)
                 CMDSTATUS_BADQUESTION, argv[0]);
     else
     {
+        char *prev = STRDUP(question_getvalue(q, ""));
         question_setvalue(q, argv[1]);
 
         if (mod->questions->methods.set(mod->questions, q) != 0)
@@ -293,15 +297,24 @@ command_set(struct confmodule *mod, char *arg)
             { /* Pass the value on to getlanguage() in templates.c */
                 debug_printf(0, "Setting debconf/language to %s", argv[1]);
                 setenv("LANGUAGE", argv[1], 1);
+                /* Reloading takes a little while, so only do it when it's
+                 * really necessary.
+                 */
+                if (!load_all_translations() &&
+                    strcmp(argv[1], "C") != 0 && strcmp(argv[1], "en") != 0 &&
+                    (prev == NULL || strcmp(argv[1], prev) != 0))
+                    mod->templates->methods.reload(mod->templates);
             }
             else if (strcmp(argv[0], "debconf/priority") == 0)
             {
                 debug_printf(0, "Setting debconf/priority to %s", argv[1]);
-                setenv("DEBCONF_PRIORITY", argv[1], 1);
+                setenv("DEBIAN_PRIORITY", argv[1], 1);
             }
         }
         else
             asprintf(&out, "%u cannot set value", CMDSTATUS_INTERNALERROR);
+
+        free(prev);
     }
     question_deref(q);
 
@@ -395,8 +408,9 @@ command_register(struct confmodule *mod, char *arg)
     }
     question_owner_add(q, mod->owner);
     q->template = t;
-    template_ref(t);
+        /* steal reference from mod->templates->methods.get above */
     mod->questions->methods.set(mod->questions, q);
+    question_deref(q);
     asprintf(&out, "%u", CMDSTATUS_SUCCESS);
     return out;
 }
@@ -418,6 +432,7 @@ command_unregister(struct confmodule *mod, char *arg)
         return out;
     }
     question_owner_delete(q, mod->owner);
+    question_deref(q);
     asprintf(&out, "%u", CMDSTATUS_SUCCESS);
 
     return out;
@@ -451,13 +466,15 @@ command_metaget(struct confmodule *mod, char *arg)
         return out;
     }
 
-    value = question_get_field(q, "", argv[1]);
+    value = question_get_raw_field(q, "", argv[1]);
     if (value == NULL)
         asprintf(&out, "%u %s does not exist", CMDSTATUS_BADQUESTION, argv[1]);
     else
         asprintf(&out, "%u %s", CMDSTATUS_SUCCESS, value);
     free(value);
 
+    question_deref(q);
+
     return out;
 }
 
@@ -571,12 +588,13 @@ command_x_loadtemplatefile(struct confmodule *mod, char *arg)
 {
     struct template *t = NULL;
     struct question *q = NULL;
-    char *argv[3];
+    char *argv[3] = { "", "", "" };
     int argc;
     char *out;
 
     argc = strcmdsplit(arg, argv, DIM(argv));
-    CHECKARGC(== 1);
+    CHECKARGC(>= 1);
+    CHECKARGC(<= 2);
     t = template_load(argv[0]);
     while (t)
     {
@@ -592,7 +610,10 @@ command_x_loadtemplatefile(struct confmodule *mod, char *arg)
             q->template = t;
             template_ref(t);
         }
+        if (*argv[1])
+            question_owner_add(q, argv[1]);
         mod->questions->methods.set(mod->questions, q);
+        question_deref(q);
         t = t->next;
     }
     asprintf(&out, "%u OK", CMDSTATUS_SUCCESS);
@@ -604,7 +625,6 @@ command_progress(struct confmodule *mod, char *arg)
 {
     int min, max;
     struct question *q = NULL;
-    char *value;
     char *argv[6];
     int argc;
     char *out;
@@ -634,16 +654,9 @@ command_progress(struct confmodule *mod, char *arg)
                     CMDSTATUS_BADQUESTION, argv[3]);
             return out;
         }
-        value = question_get_field(q, "", "description");
-        if (value == NULL)
-        {
-            asprintf(&out, "%u %s description field does not exist",
-                    CMDSTATUS_BADQUESTION, argv[3]);
-            return out;
-        }
         mod->frontend->methods.progress_start(mod->frontend,
-                min, max, value);
-        free(value);
+                min, max, q);
+        question_deref(q);
     }
     else if (strcasecmp(argv[0], "set") == 0)
     {
@@ -677,15 +690,8 @@ command_progress(struct confmodule *mod, char *arg)
                     CMDSTATUS_BADQUESTION, argv[1]);
             return out;
         }
-        value = question_get_field(q, "", "description");
-        if (value == NULL)
-        {
-            asprintf(&out, "%u %s description field does not exist",
-                    CMDSTATUS_BADQUESTION, argv[1]);
-            return out;
-        }
-        ret = mod->frontend->methods.progress_info(mod->frontend, value);
-        free(value);
+        ret = mod->frontend->methods.progress_info(mod->frontend, q);
+        question_deref(q);
         if (ret == DC_GOBACK)
         {
             asprintf(&out, "%u progress bar cancelled",
@@ -697,6 +703,12 @@ command_progress(struct confmodule *mod, char *arg)
     {
         mod->frontend->methods.progress_stop(mod->frontend);
     }
+    else
+    {
+        asprintf(&out, "%u unknown subcommand %s",
+                CMDSTATUS_SYNTAXERROR, argv[0]);
+        return out;
+    }
 
     asprintf(&out, "%u OK", CMDSTATUS_SUCCESS);
     return out;
@@ -719,7 +731,8 @@ command_settitle(struct confmodule *mod, char *arg)
         asprintf(&out, "%u %s does not exist", CMDSTATUS_BADQUESTION, arg);
 	return out;
     }
-    value = question_get_field(q, "", "description");
+    value = question_get_raw_field(q, "", "description");
+    question_deref(q);
     if (value == NULL)
     {
 	asprintf(&out, "%u %s description field does not exist",
@@ -771,6 +784,7 @@ command_info(struct confmodule *mod, char *arg)
         return out;
     }
     mod->frontend->methods.info(mod->frontend, q);
+    question_deref(q);
 
     asprintf(&out, "%u OK", CMDSTATUS_SUCCESS);
     return out;
@@ -809,11 +823,18 @@ command_data(struct confmodule *mod, char *arg)
             q->template = t;
             template_ref(t);
         }
-        t->lset(t, NULL, item, value);
+        else if (q->template != t) {
+            template_deref(q->template);
+            q->template = t;
+            template_ref(t);
+        }
+        template_lset(t, NULL, item, value);
         mod->questions->methods.set(mod->questions, q);
+        question_deref(q);
     }
     else
-        t->lset(t, NULL, item, value);
+        template_lset(t, NULL, item, value);
+    template_deref(t);
 
     asprintf(&out, "%u OK", CMDSTATUS_SUCCESS);
     return out;
